// MotorGui.java
// RoboMode with simple GUI
// Motor at port M4, end (limit) switch at I7, step switch at I8
// Show error while opening the device:
// Not connected, no power, run twice, etc.

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import ch.aplu.ftcontroller.*;

public class MotorGui extends JFrame
  implements MotorListener, ConnectionListener
{
  private FtController fc = new FtController(false);
  private JPanel panel = new JPanel();
  private JButton rightButton = new JButton("Right 40");
  private JButton leftButton = new JButton("Left 10");
  private JButton stopButton = new JButton("Stop");
  private JButton rewindButton = new JButton("Rewind");
  private JButton quitButton = new JButton("Quit");
  private Motor motor = fc.motor4;
  private volatile boolean isQuitting = false;

  public MotorGui()
  {
    panel.setPreferredSize(new Dimension(400, 40));
    getContentPane().add(panel);
    panel.add(rightButton);
    panel.add(leftButton);
    panel.add(stopButton);
    panel.add(rewindButton);
    panel.add(quitButton);
    panel.validate();
    pack();

    motor.setSpeed(6);
    fc.addMotorListener(this, motor);
    fc.addConnectionListener(this);

    ActionListener listener = new ActionListener()
    {
      public void actionPerformed(ActionEvent evt)
      {
        isQuitting = false;
        Object source = evt.getSource();
        if (source == rightButton)
        {
          motor.right(40);
          setTitle("Turning right");
        }
        if (source == leftButton)
        {
          motor.left(10);
          setTitle("Turning left");
        }
        if (source == stopButton)
        {
          motor.stop();
          setTitle("Stopped");
        }
        if (source == rewindButton)
        {
          motor.left(400);
          setTitle("Rewinding");
        }
        if (source == quitButton)
          cleanup();
      }
    };

    setTitle("Connecting...");
    boolean rc = fc.connect();
    if (!rc)
      setTitle("Connection failed");
    else
    {
      setTitle("Connected");
      rightButton.addActionListener(listener);
      leftButton.addActionListener(listener);
      stopButton.addActionListener(listener);
      rewindButton.addActionListener(listener);
    }
    quitButton.addActionListener(listener);
  }

  protected void processWindowEvent(WindowEvent e)
  {
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
      cleanup();
    else
      super.processWindowEvent(e);
  }

  public void motorStopped(int motorNb, boolean isEnd)
  {
    if (isEnd)
    {
      setTitle("Rewind position reached");
      if (isQuitting)
        fc.exit();
    }
    else
      setTitle("Step count reached");
  }

  public void stepChanged(int motorNb, int step)
  {
    setTitle("Stepping (# of steps: " + step + ")");
  }

  public void cleanup()
  {
    if (!fc.isConnected())
      fc.exit();
    isQuitting = true;
    motor.left(400);
  }

  public void connectionLost()
  {
    setTitle("Connection lost");
    disableButtons();
  }

  private void disableButtons()
  {
    rightButton.setEnabled(false);
    leftButton.setEnabled(false);
    stopButton.setEnabled(false);
    rewindButton.setEnabled(false);
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new MotorGui().setVisible(true);
      }
    });
  }
}

