// AnalogIn.java

/*
This software is part of the Aplu FtController package.
It is Open Source Free Software, so you may
- run the code for any purpose
- study how the code works and adapt it to your needs
- integrate all or parts of the code in your own programs
- redistribute copies of the code
- improve the code and release your improvements to the public
However the use of the code is entirely your responsibility.
 */

package ch.aplu.ftcontroller;

/**
 * A class the represents one of the analog input ports (A1, A2).
 */
public class AnalogIn
{
  private FtController fc;
  private int portNb;

  protected AnalogIn(FtController fc, int portNb)
  {
    this.fc = fc;
    this.portNb = portNb;
  }
  
  /**
   * Polls the port and returns its current value.
   * @return the current value (0..1023)
   */
  public int getValue()
  {
    synchronized(fc)
    {
      if (portNb == 1)
        return fc.nh.invoke(13);
      if (portNb == 2)
        return fc.nh.invoke(14);
      return -1;
    }
   }

  /**
   * Sets the jitter interval. This defines an interval around the
   * current port value. A new input value must lay outside this
   * interval in order an AnalogInput event is triggered. This may be useful
   * to suppress fluctuations due to noise.
   * @param jitter the interval to suppress noise (0..1000)
   */
  public void setJitter(int jitter)
  {
    if (jitter < 0 || jitter > 1000)
      return;
    if (portNb == 1)
      fc.a1Jitter = jitter;
    if (portNb == 2)
      fc.a2Jitter = jitter;
    synchronized(fc)
    {
      fc.nh.invoke(104);
    }
  }
}
