// Lamp.java

/*
This software is part of the Aplu FtController package.
It is Open Source Free Software, so you may
- run the code for any purpose
- study how the code works and adapt it to your needs
- integrate all or parts of the code in your own programs
- redistribute copies of the code
- improve the code and release your improvements to the public
However the use of the code is entirely your responsibility.
 */

package ch.aplu.ftcontroller;

/**
 * Class to handle the 8 lamps (or other devices) at output ports O1..O8.
 */
public class Lamp
{
  private FtController fc;
  private int lampNb;

  protected Lamp(FtController fc, int lampNb)
  {
    this.fc = fc;
    this.lampNb = lampNb;
  }

  /**
   * Returns number of lamp (1..8).
   * @return the number of the lamp
   */
  public int getNumber()
  {
    return lampNb;
  }  
 
 /** 
   * Sets the intensity of the lamp.
   * If the lamp is turned on, the intensity is modified immediately.
   * Default value: 8.
   * @param intensity the intensity value 1..8 (0 is lowest)
   * @return a reference to the motor to allow chaining
   */
   public Lamp setIntensity(int intensity)
  {
    if (intensity < 1 || intensity > 8)
      return this;
    fc.intensities[lampNb - 1] = intensity;
    fc.lampNb = lampNb;
    if (fc.isLampOn)
      on();
    return this;
  }

  /**
   * Turns the lamp on with current intensity.
   * @return a reference to the lamp to allow chaining.
   */
  public Lamp on()
  {
    fc.isLampOn = true;
    fc.lampNb = lampNb;
    synchronized(fc)
    {
      fc.nh.invoke(16);
    }
    return this;
  }
  
  /**
   * Turns the lamp off.
   * @return a reference to the lamp to allow chaining.
   */
  public Lamp off()
  {
    fc.isLampOn = false;
    fc.lampNb = lampNb;
    synchronized(fc)
    {
      fc.nh.invoke(16);
    }
    return this;
  }
}
