// FtControllerHandler.cpp

#include "FtControllerHandler.h"
#include "iostream"

using namespace std;

// #define DEBUG

// ----------------- Selector -------------- ---------------
WindowHandler *
  selectWindowHandler(JNIEnv * env, jobject obj, int select)
{
  return new FtControllerHandler(env, obj);
}

// ----------------- class FtControllerHandler ------------------
FtControllerHandler::FtControllerHandler(JNIEnv * env, jobject obj)
    : WindowHandler(env, obj), isReady(false), 
      irCode(0), irKeyNb(0), eventMask(0), clock(0), isNewEventMask(false)
{
  data = new int[19];
  newData = new int[19];
  jitter = new int[19];
  for (int i = 0; i < 17; i++)
    data[i] = newData[i] = jitter[i] = 0;
  data[17] = 1; // Indicates "data valid"
  period = JNIVar<int>(_exposedObj, "controllerPollPeriod").get();
#ifdef DEBUG
  cout << "FtController in debug mode started" << endl;
#endif
}

FtControllerHandler::~FtControllerHandler()
{
  delete [] data;
  delete [] newData;
  delete [] jitter;
}

bool FtControllerHandler::wantThreadMsg(UINT uMsg)
{
  return (uMsg == WM_THREADSTART || 
                  WM_TIMER ||
                  WM_THREADSTOP);
 }

void FtControllerHandler::evThreadMsg(HWND /*hwnd*/, 
                                      UINT message, 
                                      WPARAM /* wParam */, 
                                      LPARAM /* lParam */)
// Runs in JNIThread and not in the normal application thread
{
  #define ID_TIMER 1

  switch (message)
  {
    case WM_THREADSTART:
      SetTimer(0, ID_TIMER, period, (TIMERPROC)NULL);
      break;

    case WM_TIMER:
#ifdef DEBUG
      clock++;
      if (clock == maxInt)
        clock = 0;
#endif

      getState();
      if (isDataChanged())
        writeBuf();

      if (isNewEventMask)
      {
        isNewEventMask = false;
        eventMask = newEventMask;
      }

      if (!isReady)
        isReady = true;
      break;
    
    case WM_THREADSTOP:
      KillTimer(0, ID_TIMER);
      if (isConnected)
        rbCloseInterface(handle);
      break;
  }
}

int FtControllerHandler::invoke(int val)
{
  if (val == 0)  // ------------------ open ----------------------------------
  {
    handle = rbOpenInterfaceUSBdis(0, 0, ftiDisUltra);
    if (handle == FT_ERROR)
      isConnected = false;
    else
    {
      isConnected = true;
      int digIn  = rbGetInputs(handle);
      data[8] = newData[8] = digIn;  // Default value set to current value
    }
    return handle;
  }
  else if (val == 2) // -------------- stop ----------------------------------
  {
    if (!isConnected)
      return -1;
    int motorNb = JNIVar<int>(_exposedObj, "motorNb").get(); 
    rbSetModeStatus(handle, motorNb, 0);
   	return rbSetMotorEx(handle, motorNb, 0, 0);
  }
  else if (val >= 3 &&  val <= 6) // -------- right/left ----------------------
  {
    if (!isConnected)
      return -1;
    int motorNb = JNIVar<int>(_exposedObj, "motorNb").get(); 
    JNIArray<int> jary(_exposedObj, "speeds");
    int * speeds = jary.get();
    int speed = speeds[motorNb-1] - 1;  // Java speeds from 1..8

    if (val == 3) // --------------------------- left() -----------------------
    {
      rbSetModeStatus(handle, motorNb, 0);
      return rbSetMotorEx(handle, motorNb, 1, speed);
    }
    else if (val == 4)  // --------------------- right()----- ------------------
    {
      rbSetModeStatus(handle, motorNb, 0);
    	return rbSetMotorEx(handle, motorNb, 2, speed);
    }
    else if (val == 5) // ---------------------- left(steps) -------------------
    {
      JNIArray<int> jary(_exposedObj, "steps");
      int * steps = jary.get();
      int stepNb = steps[motorNb-1];
      rbSetModeStatus(handle, motorNb, 1);
      return rbRobMotor(handle, motorNb, 1, speed, stepNb);
    }
    else // --------------------------------------- right(steps) ---------------
    {
      JNIArray<int> jary(_exposedObj, "steps");
      int * steps = jary.get();
      int stepNb = steps[motorNb-1];
      rbSetModeStatus(handle, motorNb, 1);
      return rbRobMotor(handle, motorNb, 2, speed, stepNb);
    }
  }
  else if (val == 7) // ---------------- getCounter --------------------------
  {
    if (!isConnected)
      return -1;
    int counterNb = JNIVar<int>(_exposedObj, "counterNb").get();
    return rbGetCounter(handle, counterNb);
  }
  else if (val == 8) // ---------------- setCounter --------------------------
  {
    if (!isConnected)
      return -1;
    int counterNb = JNIVar<int>(_exposedObj, "counterNb").get();
    int count = JNIVar<int>(_exposedObj, "count").get();
    return rbSetCounter(handle, counterNb, count);
  }
  else if (val == 9) // ---------------- clearCounters -----------------------
  {
    if (!isConnected)
      return -1;
    return rbClearCounters(handle);
  }
  else if (val == 10) // ---------------- getInputs --------------------------
  {
    if (!isConnected)
      return -1;
    return rbGetInputs(handle);
  }
  else if (val == 11) // ---------------- getResistor port AX ----------------
  {
    if (!isConnected)
      return -1;
    return rbGetAnalog(handle, 1);
  }
  else if (val == 12) // ---------------- getResistor port AY ----------------
  {
    if (!isConnected)
      return -1;
    return rbGetAnalog(handle, 2);
  }
  else if (val == 13) // ---------------- getAnalog port A1 ------------------
  {
    if (!isConnected)
      return -1;
    return rbGetVoltage(handle, 1);
  }
  else if (val == 14) // ---------------- getAnalog port A2 ------------------
  {
    if (!isConnected)
      return -1;
    return rbGetVoltage(handle, 2);
  }
  else if (val == 15) // ---------------- getInfrared ------------------------
  {
    if (!isConnected)
      return -1;
    return rbGetIRKey(handle, 0, 0);
  }
  else if (val == 16) // ---------------- LampOn/Off ------------------------ 
  {
    if (!isConnected)
      return -1;
    bool isLampOn = JNIVar<bool>(_exposedObj, "isLampOn").get();
    int lampNb = JNIVar<int>(_exposedObj, "lampNb").get(); 
    JNIArray<int> jary(_exposedObj, "intensities");
    int * intensities = jary.get();
    int intensity = intensities[lampNb-1] - 1;  // Java intensities 1..8
    return rbSetLampEx(handle, lampNb, isLampOn ? 1 : 0, intensity);
  }
  else if (val == 17) // ---------------- getDeviceType ----------------------
  {
    if (!isConnected)
      return -1;
    return rbGetActDeviceType(handle);
  }
  else if (val == 18) // ---------------- getDeviceSerialNumber --------------
  {
    if (!isConnected)
      return -1;
    return rbGetActDeviceSerialNr(handle);
  }
  else if (val == 19) // ---------------- getDeviceFirmwareNumber ------------
  {
    if (!isConnected)
      return -1;
    return rbGetActDeviceFirmwareNr(handle);
  }
  else if (val == 20) // ---------------- getDeviceFirmwareInfo   ------------
  {
    if (!isConnected)
      return -1;
    const char * cstr = rbGetActDeviceFirmware(handle);
    JNIString(_exposedObj, "firmwareInfo").setMBCS(cstr);
    return 0;
  }
  else if (val == 21) // ---------------- getDeviceName ----------------------
  {
    if (!isConnected)
      return -1;
    const char * cstr = rbGetActDeviceFirmware(handle);
    JNIString(_exposedObj, "deviceName").setMBCS(cstr);
    return 0;
  }
  else if (val == 22) // ---------------- getDistance D1 ---------------------
  {
    if (!isConnected)
      return -1;
    return rbGetDistanceValue(handle, 1);
  }
  else if (val == 23) // ---------------- getDistance D2 ---------------------
  {
    if (!isConnected)
      return -1;
    return rbGetDistanceValue(handle, 2);
  }
  else if (val == 24) // ---------------- getMotors --------------------------
  {
    if (!isConnected)
      return -1;
    return rbGetMotors(handle);
  }
  else if (val == 100) // -------------- check if ready ----------------------
  {
    return isReady;
  }
  else if (val == 102) // -------------- set default digitalIn ---------------
  {
    if (!isConnected)
      return -1;
    int digIn  = rbGetInputs(handle);
    data[8] = newData[8] = digIn;
    return 0;
  }
  else if (val == 103) // -------------- event mask --------------------------
  {
    isNewEventMask = true;
    newEventMask = JNIVar<int>(_exposedObj, "eventMask").get();
#ifdef DEBUG
    cout << "New Event Mask: " << newEventMask << endl;
#endif
    setJitter();
    return 0;
  }
  else if (val == 104) // -------------- setJitter ----------------------------
  {
    setJitter();
    return 0;
  }
  return 0;
}

void FtControllerHandler::setJitter()
{
    jitter[9] = JNIVar<int>(_exposedObj, "axJitter").get();
    jitter[10] = JNIVar<int>(_exposedObj, "ayJitter").get();
    jitter[11] = JNIVar<int>(_exposedObj, "a1Jitter").get();
    jitter[12] = JNIVar<int>(_exposedObj, "a2Jitter").get();
}

void FtControllerHandler::getState()
{
  if (isConnected)
  {
    if (rbGetInputs(handle) == FT_ERROR)
    {
      isConnected = false;
      rbCloseInterface(handle);
      newData[17] = 0;
      newData[18] = 0;  // mask
      return;
    }
  }
  else
    return;

  int bitValue = 1;
  for (int i = 0; i < 8; i++)
  {
    if ((eventMask & bitValue) != 0)
      newData[i] = rbGetCounter(handle, i+1);
    bitValue <<= 1;
  }
  if ((eventMask & bitValue) != 0)
    newData[8]  = rbGetInputs(handle);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
    newData[9] = rbGetAnalog(handle, 1);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
    newData[10] = rbGetAnalog(handle, 2);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
    newData[11] = rbGetVoltage(handle, 1);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
    newData[12] = rbGetVoltage(handle, 2);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
    newData[13] = rbGetDistanceValue(handle, 1);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
    newData[14] = rbGetDistanceValue(handle, 2);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
    newData[15] = rbGetIRKey(handle, irCode, irKeyNb);
  bitValue <<= 1;
  if ((eventMask & bitValue) != 0)
  {
    newData[16] = 0;
    for (int i = 0; i < 4; i++)
      newData[16] += (1 << i) * rbGetInput(handle, 2 * i + 1);  // End switches
  }

  // Error while reading inputs
  for (int i = 0; i < 17; i++)
  {
    if (newData[i] == FT_ERROR)
    {
      isConnected = false;
      rbCloseInterface(handle);
      newData[17] = 0;
      break;
    }
    else
      newData[17] = true;
  }

  newData[18] = 0;  // mask
}

void FtControllerHandler::writeBuf()
{
#ifdef DEBUG
    cout << "Sent at " << clock << ": "; 
#endif
  for (int i = 0; i < 19; i++)
  {
    writeInt(data[i]);
#ifdef DEBUG
    cout << (unsigned int)data[i];
    if (i < 18)
      cout << ", ";
#endif
  }
#ifdef DEBUG
    cout << endl;
#endif
}

bool FtControllerHandler::isDataChanged()
{
  bool isChanged = false;
  int mask = 0;
  int currentBit = 1;
  for (int i = 0; i < 18; i++)
  {
    if (((data[i] & 0x0000FFFF) > newData[i] + jitter[i]) || 
        ((data[i] & 0x0000FFFF) < newData[i] - jitter[i]))  // mask hiword (digital-in mask)
    {
      isChanged = true;
      mask += currentBit;
      if (i == 8) // Digital-in
      {
        short dinMask = 0;
        short currentDinBit = 1;
        for (int k = 0; k < 8; k++)
        {
          if ((data[8] & currentDinBit) != (newData[8] & currentDinBit))
          { 
            data[8] = data[8] & 0x0000FFFF; // Clear digital-in mask
            data[8] = data[8] & ~currentDinBit; // Clear bit
            data[8] = data[8] | (newData[8] & currentDinBit);  // Set bit
            dinMask += currentDinBit;
            data[8] = data[8] + (dinMask << 16);  // Set digital-in mask in hiword
          }
          currentDinBit *= 2;
        }
      }
      else
        data[i] = newData[i];
    }
    currentBit *= 2;
  }
  data[18] = mask;
  return isChanged;
}
